@@ -1,12 +1,14 @@
-import uuid
+import math
 import time
-
+import uuid
 from queue import Queue, Empty
+
+from .network import Network
+from qunetsim.backends import EQSNBackend
 from qunetsim.components import protocols
-from qunetsim.utils.constants import Constants
 from qunetsim.objects import Logger, DaemonThread, Message, Packet, Qubit, QuantumStorage, ClassicalStorage, \
     QuantumConnection, ClassicalConnection
-from qunetsim.backends import EQSNBackend
+from qunetsim.utils.constants import Constants
 
 
 class Host(object):
@@ -80,7 +82,7 @@ class Host(object):
         Gets the classical connections of the host.
 
         Returns:
-            classical connections
+            (dict): classical connections
         """
         return self._classical_connections
 
@@ -228,7 +230,7 @@ class Host(object):
         Get the quantum connections for the host.
 
         Returns:
-            (list): The quantum connections for the host.
+            (dict): The quantum connections for the host.
         """
         return self._quantum_connections
 
@@ -641,8 +643,13 @@ class Host(object):
             (bool): Success status of the removal
         """
         try:
+            network = Network.get_instance()
+            network.remove_c_connection(self.host_id, receiver_id)
             del self.classical_connections[receiver_id]
             return True
+        except KeyError:
+            self.logger.error('Tried to delete a classical connection tha does not exist')
+            return False
         except NameError:
             self.logger.error('Tried to delete a classical connection tha does not exist')
             return False
@@ -657,8 +664,13 @@ class Host(object):
             (bool): Success status of the removal
         """
         try:
+            network = Network.get_instance()
+            network.remove_q_connection(self.host_id, receiver_id)
             del self.quantum_connections[receiver_id]
             return True
+        except KeyError:
+            self.logger.error('Tried to delete a quantum connection tha does not exist')
+            return False
         except NameError:
             self.logger.error('Tried to delete a quantum connection tha does not exist')
             return False
@@ -675,7 +687,10 @@ class Host(object):
         packet = protocols.encode(sender=self.host_id,
                                   receiver=receiver,
                                   protocol=Constants.SEND_CLASSICAL,
-                                  payload=Message(sender=self.host_id, content=Constants.ACK, seq_num=seq_number),
+                                  payload=Message(
+                                      sender=self.host_id,
+                                      content=Constants.ACK,
+                                      seq_num=seq_number),
                                   payload_type=Constants.SIGNAL,
                                   sequence_num=seq_number,
                                   await_ack=False)
@@ -908,7 +923,9 @@ class Host(object):
         packet = protocols.encode(sender=self.host_id,
                                   receiver=None,
                                   protocol=Constants.SEND_GHZ,
-                                  payload={Constants.QUBITS: q_list, Constants.HOSTS: receiver_list},
+                                  payload={
+                                      Constants.QUBITS: q_list,
+                                      Constants.HOSTS: receiver_list},
                                   payload_type=Constants.CLASSICAL,
                                   sequence_num=seq_num_list,
                                   await_ack=await_ack)
@@ -924,6 +941,85 @@ class Host(object):
             return q_id, ret
         return q_id
 
+    def send_w(self, receiver_list, q_id=None, await_ack=False, no_ack=False, distribute=False):
+        """
+        Share W state with all receiver ids in the list. W state is generated
+        locally.
+
+        Args:
+            receiver_list (list): A List of all Host IDs with which a W state
+                                  should be shared.
+            q_id (str): The ID of the W qubits
+            await_ack (bool): If the sender should await an ACK from all receivers
+            no_ack (bool): If this message should not use any ACK and sequencing.
+            distribute (bool): If the sender should keep part of the W state, or just
+                               distribute one
+        Returns:
+            (str, bool): Qubit ID of the shared W and ACK status
+        """
+
+        q_list = []
+        n = len(receiver_list) + (0 if distribute else 1)
+
+        def f_gate(i, j, k):
+            theta = math.acos(math.sqrt(1 / (n - k + 1)))
+            q_list[j].ry(-theta)
+            q_list[i].cphase(q_list[j])  # CZ gate
+            q_list[j].ry(theta)
+            return
+
+        own_qubit = Qubit(self, q_id=q_id)
+        q_list.append(own_qubit)
+        q_id = own_qubit.id
+
+        # create a qubit for each participant to the W state
+        for _ in range(n - 1):
+            new_qubit = Qubit(self, q_id=q_id)
+            q_list.append(new_qubit)
+
+        # creating the W state
+        q_list[n - 1].X()
+        for idx in range(n - 1):
+            f_gate(n - 1 - idx, n - 2 - idx, idx + 1)
+
+        for idx in range(n - 1):
+            q_list[n - 2 - idx].cnot(q_list[n - 1 - idx])
+
+        q_list = q_list[1:] + q_list[:1]
+        seq_num_list = []
+
+        if not distribute:
+            self.add_w_qubit(self.host_id, own_qubit)
+
+        for receiver_id in receiver_list:
+            seq_num = -1
+            if no_ack:
+                await_ack = False
+            else:
+                seq_num = self.get_next_sequence_number(receiver_id)
+            seq_num_list.append(seq_num)
+
+        packet = protocols.encode(sender=self.host_id,
+                                  receiver=None,
+                                  protocol=Constants.SEND_W,
+                                  payload={
+                                      Constants.QUBITS: q_list,
+                                      Constants.HOSTS: receiver_list},
+                                  payload_type=Constants.CLASSICAL,
+                                  sequence_num=seq_num_list,
+                                  await_ack=await_ack)
+        self.logger.log(self.host_id + " sends W to " + str(receiver_list))
+        self._packet_queue.put(packet)
+
+        if packet.await_ack:
+            ret = True
+            for receiver_id, seq_num in zip(receiver_list, seq_num_list):
+                self._log_ack('W', receiver_id, seq_num)
+                if self.await_ack(seq_num, receiver_id) is False:
+                    ret = False
+            return q_id, ret
+        return q_id
+
     def get_ghz(self, host_id, q_id=None, wait=0):
         """
         Gets the GHZ qubit which has been created by the host with the host ID *host_id*.
@@ -941,7 +1037,25 @@ class Host(object):
 
         return _get_qubit(self._qubit_storage, host_id, q_id, Qubit.GHZ_QUBIT, wait)
 
-    def send_teleport(self, receiver_id, q, await_ack=False, no_ack=False, payload=None, generate_epr_if_none=True):
+    def get_w(self, host_id, q_id=None, wait=0):
+        """
+        Gets the W qubit which has been created by the host with the host ID *host_id*.
+        It is not necessary to know with whom the states are shared.
+
+        Args:
+            host_id (str): The ID of the host that creates the GHZ state.
+            q_id (str): The qubit ID of the W to get.
+            wait (float): the amount of time to wait
+        Returns:
+             (Qubit): Qubit shared with the host with *host_id* and *q_id*.
+        """
+        if not isinstance(wait, float) and not isinstance(wait, int):
+            raise Exception('wait parameter should be a number')
+
+        return _get_qubit(self._qubit_storage, host_id, q_id, Qubit.W_QUBIT, wait)
+
+    def send_teleport(self, receiver_id, q, await_ack=False, no_ack=False,
+                      payload=None, generate_epr_if_none=True):
         """
         Teleports the qubit *q* with the receiver with host ID *receiver*
 
@@ -1098,20 +1212,50 @@ class Host(object):
             raise ValueError("Host id has to be specified!")
         return self._qubit_storage.get_all_qubits_from_host(host_id, Qubit.EPR_QUBIT)
 
-    def get_data_qubits(self, host_id):
+    def get_data_qubits(self, host_id, remove_from_storage=False):
         """
         Return the dictionary of data qubits stored, just for the information regarding which qubits are stored.
-        Does not remove the qubits from storage like *get_data_qubit* does.
+        Optional to remove the qubits from storage like *get_data_qubit* does with *remove_from_storage* field.
 
         Args:
             host_id (str): The host id from which the data qubit have been received.
+            remove_from_storage (bool): Get and remove from storage.
 
         Returns:
             (dict): If *host_id* is not set, then return the entire dictionary of data qubits.
                   Else If *host_id* is set, then return the data qubits for that particular host if there are any.
                   Return an empty list otherwise.
         """
-        return self._qubit_storage.get_all_qubits_from_host(host_id, Qubit.DATA_QUBIT)
+        return self._qubit_storage.get_all_qubits_from_host(host_id,
+                                                            purpose=Qubit.DATA_QUBIT,
+                                                            remove=remove_from_storage)
+
+    def reset_data_qubits(self, host_id=None):
+        """
+        Remove all qubits associated with *host_id* else remove all qubits if *host_id* not set.
+
+        Args:
+            host_id (str): The host ID to remove associated qubits with.
+
+        """
+        if host_id is not None:
+            self._qubit_storage.reset_qubits_from_host(host_id)
+        else:
+            # Remove all qubits
+            for q_conn in self._quantum_connections.keys():
+                self._qubit_storage.reset_qubits_from_host(q_conn, purpose=Qubit.DATA_QUBIT)
+
+    def get_number_of_data_qubits(self, host_id):
+        """
+        Return the number of data qubits associated with host *host_id*.
+
+        Args:
+            host_id (str):
+
+        Returns:
+            (int):
+        """
+        return self._qubit_storage.amount_qubits_stored_with_host(host_id)
 
     def set_epr_memory_limit(self, limit, host_id=None):
         """
@@ -1147,7 +1291,7 @@ class Host(object):
         otherwise generates an ID for the EPR and adds the qubit with that ID.
 
         Args:
-            host_id (String): The ID of the host to pair the qubit
+            host_id (str): The ID of the host to pair the qubit
             qubit (Qubit): The data Qubit to be added.
             q_id (str): The ID of the qubit to be added.
             blocked (bool): If the qubit should be stored as blocked or not
@@ -1196,6 +1340,24 @@ class Host(object):
         self._qubit_storage.add_qubit_from_host(qubit, Qubit.GHZ_QUBIT, host_id)
         return qubit.id
 
+    def add_w_qubit(self, host_id, qubit, q_id=None):
+        """
+        Adds the W qubit to the storage of the host. The host id corresponds
+        to the generator of the W state.
+
+        Args:
+            host_id (str): The ID of the host to pair the qubit
+            qubit (Qubit): The data Qubit to be added.
+            q_id (str): the ID to set the qubit ID to
+        Returns:
+            (str): The qubit ID
+        """
+        if q_id is not None:
+            qubit.id = q_id
+
+        self._qubit_storage.add_qubit_from_host(qubit, Qubit.W_QUBIT, host_id)
+        return qubit.id
+
     def add_checksum(self, qubits, size_per_qubit=2):
         """
         Generate a set of qubits that represent a quantum checksum for the set of qubits *qubits*
