@@ -1,13 +1,14 @@
-import networkx as nx
-import matplotlib.pyplot as plt
-import time
 import random
-
-from qunetsim.objects import Qubit, RoutingPacket, Logger, DaemonThread
-from queue import Queue
-from qunetsim.utils.constants import Constants
+import time
 from inspect import signature
+from queue import Queue
+
+import matplotlib.pyplot as plt
+import networkx as nx
+
 from qunetsim.backends import EQSNBackend
+from qunetsim.objects import Qubit, RoutingPacket, Logger, DaemonThread
+from qunetsim.utils.constants import Constants
 
 
 # Network singleton
@@ -21,6 +22,13 @@ class Network:
             Network()
         return Network.__instance
 
+    @staticmethod
+    def reset_network():
+        if Network.__instance is not None:
+            Network.__instance.stop(True)
+            Network.__instance = None
+        __instance = Network()
+
     def __init__(self):
         if Network.__instance is None:
             self.ARP = {}
@@ -107,13 +115,15 @@ class Network:
             algorithm (function): The routing algorithm of the network. Should take an input and an output
         """
         if not callable(algorithm):
-            raise Exception("The quantum routing algorithm must be a function.")
+            raise Exception(
+                "The quantum routing algorithm must be a function.")
 
         num_algo_params = len(signature(algorithm).parameters)
         if num_algo_params != 3:
-            raise Exception("The quantum routing algorithm function should take three parameters: " +
-                            "the (nx) graph representation of the network, the sender address and the " +
-                            "receiver address.")
+            raise Exception(
+                "The quantum routing algorithm function should take three parameters: " +
+                "the (nx) graph representation of the network, the sender address and the " +
+                "receiver address.")
 
         self._quantum_routing_algo = algorithm
 
@@ -164,6 +174,14 @@ class Network:
 
         self._packet_drop_rate = drop_rate
 
+    @property
+    def arp(self):
+        return self.ARP
+
+    @property
+    def num_hosts(self):
+        return len(self.arp.keys())
+
     def add_host(self, host):
         """
         Adds the *host* to ARP table and updates the network graph.
@@ -188,7 +206,7 @@ class Network:
 
     def remove_host(self, host):
         """
-        Removes the host from the ARP table.
+        Removes the host from the network.
 
         Args:
             host (Host): The host to be removed from the network.
@@ -196,6 +214,22 @@ class Network:
 
         if host.host_id in self.ARP:
             del self.ARP[host.host_id]
+            if self.quantum_network.has_node(host.host_id):
+                self.quantum_network.remove_node(host.host_id)
+            if self.classical_network.has_node(host.host_id):
+                self.classical_network.remove_node(host.host_id)
+
+    def remove_c_connection(self, sender, receiver):
+        if self.classical_network.has_edge(sender, receiver):
+            self.classical_network.remove_edge(sender, receiver)
+
+    def remove_q_connection(self, sender, receiver):
+        if self.quantum_network.has_edge(sender, receiver):
+            self.quantum_network.remove_edge(sender, receiver)
+
+    def remove_hosts(self, hosts):
+        for host in hosts:
+            self.remove_host(host)
 
     def update_host(self, host):
         """
@@ -220,7 +254,8 @@ class Network:
         try:
             self.classical_network.remove_node(host.host_id)
         except nx.NetworkXError:
-            Logger.get_instance().error('attempted to remove a non-exiting node from network')
+            Logger.get_instance().error(
+                'attempted to remove a non-exiting node from network')
 
     def _update_network_graph(self, host):
         """
@@ -229,8 +264,11 @@ class Network:
         Args:
             host: The host to be added
         """
-        self.classical_network.add_node(host.host_id)
-        self.quantum_network.add_node(host.host_id)
+        if not self.classical_network.has_node(host.host_id):
+            self.classical_network.add_node(host.host_id)
+
+        if not self.quantum_network.has_node(host.host_id):
+            self.quantum_network.add_node(host.host_id)
 
         for connection in host.classical_connections:
             if not self.classical_network.has_edge(host.host_id, connection):
@@ -255,7 +293,8 @@ class Network:
         """
         host_sender = self.get_host(sender)
         host_receiver = self.get_host(receiver)
-        return host_sender.shares_epr(receiver) and host_receiver.shares_epr(sender)
+        return host_sender.shares_epr(receiver) and host_receiver.shares_epr(
+            sender)
 
     def get_host(self, host_id):
         """
@@ -317,9 +356,11 @@ class Network:
         Returns:
             route (list): An ordered list of ID numbers on the shortest path from source to destination.
         """
-        return self.classical_routing_algo(self.classical_network, source, dest)
+        return self.classical_routing_algo(self.classical_network, source,
+                                           dest)
 
-    def _entanglement_swap(self, sender, receiver, route, q_id, o_seq_num, blocked):
+    def _entanglement_swap(self, sender, receiver, route, q_id, o_seq_num,
+                           blocked):
         """
         Performs a chain of entanglement swaps with the hosts between sender and receiver to create a shared EPR pair
         between sender and receiver.
@@ -344,7 +385,8 @@ class Network:
         # Create EPR pairs on the route, where all EPR qubits have the id q_id
         threads = []
         for i in range(len(route) - 1):
-            threads.append(DaemonThread(establish_epr, args=(self, route[i], route[i + 1])))
+            threads.append(DaemonThread(establish_epr,
+                                        args=(self, route[i], route[i + 1])))
 
         for t in threads:
             t.join()
@@ -374,13 +416,15 @@ class Network:
                         'o_seq_num': o_seq_num,
                         'type': Constants.EPR}
 
-            host.send_teleport(route[i + 2], None, await_ack=True, payload=data, generate_epr_if_none=False)
+            host.send_teleport(route[i + 2], None, await_ack=True,
+                               payload=data, generate_epr_if_none=False)
 
         # Change in the storage that the EPR qubit is shared with the receiver
         q2 = host_sender.get_epr(route[1], q_id=q_id)
         host_sender.add_epr(receiver, q2, q_id, blocked)
-        Logger.get_instance().log('Entanglement swap was successful for pair with id '
-                                  + q_id + ' between ' + sender + ' and ' + receiver)
+        Logger.get_instance().log(
+            'Entanglement swap was successful for pair with id '
+            + q_id + ' between ' + sender + ' and ' + receiver)
 
     def _establish_epr(self, sender, receiver, q_id, o_seq_num, blocked):
         """
@@ -418,31 +462,38 @@ class Network:
             for q in qubits:
                 # Modify the qubit according to error function of the model
                 qubit_id = q.id
-                q = self.ARP[s].quantum_connections[self.ARP[r].host_id].model.qubit_func(q)
+                q = self.ARP[s].quantum_connections[
+                    self.ARP[r].host_id].model.qubit_func(q)
 
                 if q is None:
                     # Log failure of transmission if qubit is lost
-                    Logger.get_instance().log('transfer qubits - transfer of qubit ' + qubit_id + ' failed')
+                    Logger.get_instance().log(
+                        'transfer qubits - transfer of qubit ' + qubit_id + ' failed')
                     return False
 
                 else:
-                    Logger.get_instance().log('transfer qubits - sending qubit ' + q.id)
+                    Logger.get_instance().log(
+                        'transfer qubits - sending qubit ' + q.id)
 
                     q.send_to(self.ARP[r].host_id)
-                    Logger.get_instance().log('transfer qubits - received ' + q.id)
+                    Logger.get_instance().log(
+                        'transfer qubits - received ' + q.id)
 
                     # Unblock qubits in case they were blocked
                     q.blocked = False
 
                     if self.ARP[r].q_relay_sniffing:
-                        self.ARP[r].q_relay_sniffing_fn(original_sender, receiver, q)
+                        self.ARP[r].q_relay_sniffing_fn(original_sender,
+                                                        receiver, q)
             return True
 
         route = self.get_quantum_route(sender, receiver)
         i = 0
         while i < len(route) - 1:
-            Logger.get_instance().log('sending qubits from ' + route[i] + ' to ' + route[i + 1])
-            if not transfer_qubits(route[i + 1], route[i], original_sender=route[0]):
+            Logger.get_instance().log(
+                'sending qubits from ' + route[i] + ' to ' + route[i + 1])
+            if not transfer_qubits(route[i + 1], route[i],
+                                   original_sender=route[0]):
                 return False
             i += 1
         return True
@@ -476,7 +527,8 @@ class Network:
             sender, receiver = packet.sender, packet.receiver
 
             if packet.payload_type == Constants.QUANTUM:
-                if not self._route_quantum_info(sender, receiver, [packet.payload]):
+                if not self._route_quantum_info(sender, receiver,
+                                                [packet.payload]):
                     continue
 
             try:
@@ -526,9 +578,11 @@ class Network:
                         self.ARP[route[1]].rec_packet(network_packet)
 
             except nx.NodeNotFound:
-                Logger.get_instance().error("route couldn't be calculated, node doesn't exist")
+                Logger.get_instance().error(
+                    "route couldn't be calculated, node doesn't exist")
             except ValueError:
-                Logger.get_instance().error("route couldn't be calculated, value error")
+                Logger.get_instance().error(
+                    "route couldn't be calculated, value error")
             except Exception as e:
                 Logger.get_instance().error('Error in network: ' + str(e))
 
@@ -576,16 +630,18 @@ class Network:
         """
         Draws a plot of the network.
         """
-        nx.draw_networkx(self.classical_network, pos=nx.spring_layout(self.classical_network),
-                         with_labels=True, hold=False)
+        nx.draw_networkx(self.classical_network,
+                         pos=nx.spring_layout(self.classical_network),
+                         with_labels=True)
         plt.show()
 
     def draw_quantum_network(self):
         """
         Draws a plot of the network.
         """
-        nx.draw_networkx(self.quantum_network, pos=nx.spring_layout(self.quantum_network),
-                         with_labels=True, hold=False)
+        nx.draw_networkx(self.quantum_network,
+                         pos=nx.spring_layout(self.quantum_network),
+                         with_labels=True)
         plt.show()
 
     def _encode(self, route, payload, ttl=10):
@@ -603,7 +659,8 @@ class Network:
             RoutingPacket: Encoded RELAY packet
         """
         if payload.protocol != Constants.RELAY:
-            packet = RoutingPacket(route[1], '', Constants.RELAY, Constants.SIGNAL,
+            packet = RoutingPacket(route[1], '', Constants.RELAY,
+                                   Constants.SIGNAL,
                                    payload, ttl, route)
         else:
             packet = payload
@@ -615,3 +672,121 @@ class Network:
             packet.receiver = route[2]
 
         return packet
+
+    @staticmethod
+    def _get_star_topology_graph(hosts):
+        return nx.star_graph(hosts)
+
+    @staticmethod
+    def _get_ring_topology_graph(hosts):
+        graph = nx.path_graph(hosts)
+        graph.add_edge(hosts[0], hosts[len(hosts) - 1])
+        return graph
+
+    @staticmethod
+    def _get_mesh_topology_graph(hosts):
+        return nx.complete_graph(hosts)
+
+    @staticmethod
+    def _get_linear_topology_graph(hosts):
+        return nx.path_graph(hosts)
+
+    @staticmethod
+    def _get_tree_topology_graph(hosts):
+        graph = nx.empty_graph(hosts)
+        for i in range(0, len(hosts)):
+            if 2 * i + 1 < len(hosts):
+                graph.add_edge(hosts[i], hosts[2 * i + 1])
+            if 2 * i + 2 < len(hosts):
+                graph.add_edge(hosts[i], hosts[2 * i + 2])
+        return graph
+
+    topologies = {
+        'mesh': lambda x: Network._get_mesh_topology_graph(x),
+        'ring': lambda x: Network._get_ring_topology_graph(x),
+        'star': lambda x: Network._get_star_topology_graph(x),
+        'linear': lambda x: Network._get_linear_topology_graph(x),
+        'tree': lambda x: Network._get_tree_topology_graph(x)
+        # TODO: add bus topology
+    }
+
+    @staticmethod
+    def _validate_topology_input(host_names, topology):
+        if len(host_names) < 2:
+            raise ValueError('insufficient hosts')
+        if topology not in Network.topologies.keys():
+            raise ValueError('topology not implemented')
+
+    def _simple_topology_generation(self, host_names, topology, function):
+        from qunetsim import Host
+
+        Network._validate_topology_input(host_names, topology)
+
+        graph = Network.topologies[topology](host_names)
+
+        for host_name in host_names:
+            if host_name not in self.ARP.keys():
+                self.add_host(Host(host_name))
+
+        for node, adj_list in graph.adjacency():
+            h = self.get_host(node)
+            function(h, adj_list.keys())
+            self.add_host(h)
+            h.start()
+
+    def generate_topology(self, host_names: list, topology: str = 'mesh') \
+            -> None:
+        """
+        Adjusts a network that already exists by adding both classic and
+        quantum connections that fit the desired network topology. This method
+        either adds new nodes or adds connections to existing hosts within
+        the network.
+
+        The supported network topologies are mesh, star, ring, linear, and
+        tree.
+
+        Args:
+            host_names (list): The names for the new hosts.
+            topology (str) {'mesh', 'star', 'ring', 'linear', 'tree'}: The
+                network topology that will be created.
+        """
+        self.generate_c_topology(host_names, topology)
+        self.generate_q_topology(host_names, topology)
+
+    def generate_q_topology(self, host_names: list, topology: str = 'mesh') \
+            -> None:
+        """
+        Adjusts a network that already exists by adding  quantum connections
+        that fit the desired network topology. This method either adds new
+        nodes or adds connections to existing hosts within the network.
+
+        The supported network topologies are mesh, star, ring, linear, and
+        tree.
+
+        Args:
+            host_names (list): The names for the new hosts.
+            topology (str) {'mesh', 'star', 'ring', 'linear', 'tree'}: The
+                network topology that will be created.
+        """
+        def add_quantum(x, y):
+            return x.add_q_connections(y)
+        self._simple_topology_generation(host_names, topology, add_quantum)
+
+    def generate_c_topology(self, host_names: list, topology: str = 'mesh') \
+            -> None:
+        """
+        Adjusts a network that already exists by adding  quantum connections
+        that fit the desired network topology. This method either adds new
+        nodes or adds connections to existing hosts within the network.
+
+        The supported network topologies are mesh, star, ring, linear, and
+        tree.
+
+        Args:
+            host_names (list): The names for the new hosts.
+            topology (str) {'mesh', 'star', 'ring', 'linear', 'tree'}: The
+                network topology that will be created.
+        """
+        def add_classic(x, y):
+            return x.add_c_connections(y)
+        self._simple_topology_generation(host_names, topology, add_classic)
