@@ -1,5 +1,5 @@
-from qunetsim.objects.storage import ClassicalStorage, QuantumStorage
-from qunetsim.objects.connections import QuantumConnection, ClassicalConnection
+from .storage import ClassicalStorage, QuantumStorage
+from .connections import QuantumConnection, ClassicalConnection
 from .message import Message
 from .qubit import Qubit
 from .daemon_thread import DaemonThread
