@@ -1,4 +1,4 @@
-from qunetsim.objects.connections.channel_models.binary_erasure import BinaryErasure
+from qunetsim.objects.connections.channel_models.fibre import Fibre
 from qunetsim.objects.connections.connection import Connection
 
 
@@ -9,6 +9,6 @@ class QuantumConnection(Connection):
 
     def __init__(self, sender_id, receiver_id, model=None):
         if model is None:
-            super().__init__(sender_id, receiver_id, BinaryErasure())
+            super().__init__(sender_id, receiver_id, Fibre())
         else:
             super().__init__(sender_id, receiver_id, model)
